"""
exploits plugin:


"""
import logging

import click

logger = logging.getLogger("griffon")

api_url = ""


# define autocomplete for service names
@click.group(help="(UNDER DEV) exploits plugin")
@click.pass_context
def plugins(ctx):
    """exploits plugin"""
    pass


@plugins.command()
@click.option("--name", "service_name")
@click.pass_context
def stub(ctx, service_name):
    """ """
    pass
